@@ -365,6 +365,8 @@ pub fn generate(
                                     path_node: ctx_selection_set.path_node.as_ref().unwrap(),
                                     parent_type: #gql_typename,
                                     return_type: &<<#stream_ty as #crate_name::futures_util::stream::Stream>::Item as #crate_name::Type>::qualified_type_name(),
+                                    name: field.node.name.node.as_str(),
+                                    alias: field.node.alias.as_ref().map(|alias| alias.node.as_str()),
                                 };
                                 let resolve_fut = async {
                                     #crate_name::OutputType::resolve(&msg, &ctx_selection_set, &*field)
